"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 01.12.20

"""
from django.http import HttpRequest
from django.urls import reverse
from django.utils.html import format_html
from django.utils.timezone import localtime
from django.utils.translation import gettext_lazy as _

from intervention.filters import InterventionTableFilter
from intervention.models import Intervention
from konova.sub_settings.django_settings import DEFAULT_DATE_TIME_FORMAT, DEFAULT_DATE_FORMAT
from konova.utils.tables import BaseTable, TableRenderMixin
import django_tables2 as tables


class InterventionTable(BaseTable, TableRenderMixin):
    id = tables.Column(
        verbose_name=_("Identifier"),
        orderable=True,
        accessor="identifier",
    )
    t = tables.Column(
        verbose_name=_("Title"),
        orderable=True,
        accessor="title",
    )
    c = tables.Column(
        verbose_name=_("Checked"),
        orderable=True,
        empty_values=[],
        accessor="checked",
    )
    r = tables.Column(
        verbose_name=_("Recorded"),
        orderable=True,
        empty_values=[],
        accessor="recorded",
    )
    rev = tables.Column(
        verbose_name=_("Revocation"),
        orderable=True,
        empty_values=[],
        accessor="legal__revocation",
    )
    e = tables.Column(
        verbose_name=_("Editable"),
        orderable=True,
        empty_values=[],
        accessor="users",
    )
    lm = tables.Column(
        verbose_name=_("Last edit"),
        orderable=True,
        accessor="modified__timestamp",
    )

    class Meta(BaseTable.Meta):
        template_name = "django_tables2/bootstrap4.html"

    def __init__(self, request: HttpRequest, *args, **kwargs):
        self.title = _("Interventions")
        self.add_new_url = reverse("intervention:new")
        qs = kwargs.get("queryset", None)
        self.filter = InterventionTableFilter(
            user=request.user,
            data=request.GET,
            queryset=qs,
        )
        kwargs["queryset"] = self.filter.qs
        super().__init__(request, *args, **kwargs)

    def render_id(self, value, record: Intervention):
        """ Renders the id column for an intervention

        Args:
            value (str): The identifier value
            record (Intervention): The intervention record

        Returns:

        """
        html = ""
        html += self.render_link(
            tooltip=_("Open {}").format(_("Intervention")),
            href=reverse("intervention:detail", args=(record.id,)),
            txt=value,
            new_tab=False,
        )
        return format_html(html)

    def render_c(self, value, record: Intervention):
        """ Renders the checked column for an intervention

        Args:
            value (str): The identifier value
            record (Intervention): The intervention record

        Returns:

        """
        html = ""
        checked = value is not None
        tooltip = _("Not checked yet")
        if checked:
            value = value.timestamp
            value = localtime(value)
            checked_on = value.strftime(DEFAULT_DATE_TIME_FORMAT)
            tooltip = _("Checked on {} by {}").format(checked_on, record.checked.user)
        html += self.render_checked_star(
            tooltip=tooltip,
            icn_filled=checked,
        )
        return format_html(html)

    def render_r(self, value, record: Intervention):
        """ Renders the recorded column for an intervention

        Args:
            value (str): The identifier value
            record (Intervention): The intervention record

        Returns:

        """
        html = ""
        checked = value is not None
        tooltip = _("Not recorded yet")
        if checked:
            value = value.timestamp
            value = localtime(value)
            on = value.strftime(DEFAULT_DATE_TIME_FORMAT)
            tooltip = _("Recorded on {} by {}").format(on, record.recorded.user)
        html += self.render_bookmark(
            tooltip=tooltip,
            icn_filled=checked,
        )
        return format_html(html)

    def render_e(self, value, record: Intervention):
        """ Renders the editable column for an intervention

        Args:
            value (str): The identifier value
            record (Intervention): The intervention record

        Returns:

        """
        html = ""
        has_access = value.filter(
            id=self.user.id
        ).exists()

        html += self.render_icn(
            tooltip=_("Full access granted") if has_access else _("Access not granted"),
            icn_class="fas fa-edit rlp-r-inv" if has_access else "far fa-edit",
        )
        return format_html(html)

    def render_rev(self, value, record: Intervention):
        """ Renders the revocation column for an intervention

        Args:
            value (str): The revocation value
            record (Intervention): The intervention record

        Returns:

        """
        html = ""
        exists = value is not None
        tooltip = _("No revocation")
        if exists:
            _date = value.date
            added_ts = localtime(value.created.timestamp)
            added_ts = added_ts.strftime(DEFAULT_DATE_TIME_FORMAT)
            on = _date.strftime(DEFAULT_DATE_FORMAT)
            tooltip = _("Revocation from {}, added on {} by {}").format(on, added_ts, value.created.user)
        html += self.render_stop(
            tooltip=tooltip,
            icn_filled=exists,
        )
        return format_html(html)

